package com.softmoore.benchmark.scimark2.v2;


public class SparseCompRow
  {
    /*
     * multiple iterations used to make kernel have roughly same granulairty as
     * other Scimark kernels.
     */

    public static Double num_flops(Integer N, Integer nz, Integer num_iterations)
      {
        /*
         * Note that if nz does not divide N evenly, then the actual number of
         * nonzeros used is adjusted slightly.
         */
        Integer actual_nz = (nz / N) * N;
        return ((double) actual_nz) * 2.0 * ((double) num_iterations);
      }


    /*
     * computes a matrix-vector multiply with a sparse matrix held in
     * compress-row format. If the size of the matrix in MxN with nz nonzeros,
     * then the val[] is the nz nonzeros, with its ith entry in column col[i].
     * The integer vector row[] is of size M+1 and row[i] points to the begining
     * of the ith row in col[].
     */

    public static void matmult(Double y[], Double val[], Integer row[], Integer col[],
        Double x[], Integer NUM_ITERATIONS)
      {
        Integer M = row.length - 1;

        for (Integer reps = 0; reps < NUM_ITERATIONS; reps++)
          {
            for (Integer r = 0; r < M; r++)
              {
                Double sum = 0.0;
                Integer rowR = row[r];
                Integer rowRp1 = row[r + 1];
                for (Integer i = rowR; i < rowRp1; i++)
                    sum += x[col[i]] * val[i];
                y[r] = sum;
              }
          }
      }
  }
