package com.softmoore.math;


public class MatrixUtil
  {
    public static double[][] multiply(double[][] a, double[][] b)
      {
        if (!checkArgs(a, b))
            throw new IllegalArgumentException("Matrices not compatible for multiplication");
        
        int nRows = a.length;
        int nCols = b[0].length;
        
        double[][] result = new double[nRows][nCols];
        
        for (int rowNum = 0;  rowNum < nRows;  ++rowNum)
          {
            for (int colNum = 0;  colNum < nCols;  ++colNum)
              {
                double sum = 0.0;
                
                for (int i = 0;  i < a[0].length;  ++i)
                    sum += a[rowNum][i]*b[i][colNum];
                
                result[rowNum][colNum] = sum;
              }
          }
        
        return result;
      }
    
    
    public static Double[][] multiply(Double[][] a, Double[][] b)
      {
        if (!checkArgs(a, b))
            throw new IllegalArgumentException("Matrices not compatible for multiplication");
        
        int nRows = a.length;
        int nCols = b[0].length;
        
        Double[][] result = new Double[nRows][nCols];
        
        for (int rowNum = 0;  rowNum < nRows;  ++rowNum)
          {
            for (int colNum = 0;  colNum < nCols;  ++colNum)
              {
                Double sum = 0.0;
                
                for (int i = 0;  i < a[0].length;  ++i)
                    sum += a[rowNum][i]*b[i][colNum];
                
                result[rowNum][colNum] = sum;
              }
          }
        
        return result;
      }
    
    
    /**
     * Checks that the two specified matrices are compatible for multiplication.
     * @return true if the matrix product a*b is possible;
     */
    private static boolean checkArgs(double[][] a, double[][] b)
      {
        // check that matrices are not null
        if (a == null || b ==  null)
            return false;
        
        // check that we actually have columns
        if (a[0] == null || b[0] == null)
            return false;

        // check that all columns of matrix a have the same size
        int aColSize = a[0].length;
        for (int i = 1;  i < a.length;  ++i)
          {
            if (a[i] == null || a[i].length != aColSize)
                return false;
          }

        // check that all columns of matrix b have the same size
        int bColSize = b[0].length;
        for (int i = 1;  i < b.length;  ++i)
          {
            if (b[i] == null || b[i].length != bColSize)
                return false;
          }

        // check that number of columns of matrix a equals the number of rows of matrix b
        if (aColSize != b.length)
            return false;

        return true;
      }


    /**
     * Checks that the two specified matrices are compatible for multiplication.
     * @return true if the matrix product a*b is possible;
     */
    private static boolean checkArgs(Double[][] a, Double[][] b)
      {
        // check that matrices are not null
        if (a == null || b ==  null)
            return false;
        
        // check that we actually have columns
        if (a[0] == null || b[0] == null)
            return false;

        // check that all columns of matrix a have the same size
        int aColSize = a[0].length;
        for (int i = 1;  i < a.length;  ++i)
          {
            if (a[i] == null || a[i].length != aColSize)
                return false;
          }

        // check that all columns of matrix b have the same size
        int bColSize = b[0].length;
        for (int i = 1;  i < b.length;  ++i)
          {
            if (b[i] == null || b[i].length != bColSize)
                return false;
          }

        // check that number of columns of matrix a equals the number of rows of matrix b
        if (aColSize != b.length)
            return false;

        return true;
      }
  }
