package test;

/**
 * Tests the number of bytes using primitives versus the number of bytes
 * using wrapper classes. 
 */
public class TestMatrixMemoryUse
  {
    public static void main(String[] args)
      {
        int n = 1000;

        long bytesUsingWrappers   = getBytesUsingWrappers(n);
        long bytesUsingPrimitives = getBytesUsingPrimitives(n);

        System.out.println("Using wrappers:");
        System.out.println("- total bytes = " + bytesUsingWrappers);
        System.out.println("- bytes per element = " + ((double) bytesUsingWrappers)/(n*n));
        System.out.println();
        System.out.println("Using primitives:");
        System.out.println("- total bytes = " + bytesUsingPrimitives);
        System.out.println("- bytes per element = " + ((double) bytesUsingPrimitives)/(n*n));
      }


    /**
     * Returns the number of bytes used by an n-by-n matrix of type double.
     * @param  the number of rows and columns of the matrix 
     */
    public static long getBytesUsingPrimitives(int n)
      {
        System.gc();   // force garbage collection
        long memStart = Runtime.getRuntime().freeMemory();
        double[][] a = new double[n][n];

        // put some random values in the matrix
        for (int i = 0;  i < n;  ++i)
          {
            for (int j = 0; j < n;  ++j)
                a[i][j] = Math.random();
          }

        long memEnd = Runtime.getRuntime().freeMemory();

        return memStart - memEnd;
      }


    /**
     * Returns the number of bytes used by an n-by-n matrix of type Double.
     * @param  the number of rows and columns of the matrix 
     */
    public static long getBytesUsingWrappers(int n)
      {
        System.gc();   // force garbage collection
        long memStart = Runtime.getRuntime().freeMemory();
        Double[][] a = new Double[n][n];

        // put some random values in the matrix
        for (int i = 0;  i < n;  ++i)
          {
            for (int j = 0; j < n;  ++j)
                a[i][j] = Math.random();
          }

        long memEnd = Runtime.getRuntime().freeMemory();

        return memStart - memEnd;
      }
  }
